package xyz.lychee.lagfixer.libs.adventure.text.serializer.legacy;

import java.util.function.Consumer;
import java.util.regex.Pattern;
import xyz.lychee.lagfixer.libs.adventure.builder.AbstractBuilder;
import xyz.lychee.lagfixer.libs.adventure.text.Component;
import xyz.lychee.lagfixer.libs.adventure.text.TextComponent;
import xyz.lychee.lagfixer.libs.adventure.text.flattener.ComponentFlattener;
import xyz.lychee.lagfixer.libs.adventure.text.format.Style;
import xyz.lychee.lagfixer.libs.adventure.text.serializer.ComponentSerializer;
import xyz.lychee.lagfixer.libs.adventure.util.Buildable;
import xyz.lychee.lagfixer.libs.adventure.util.PlatformAPI;
import xyz.lychee.lagfixer.libs.annotations.NotNull;
import xyz.lychee.lagfixer.libs.annotations.Nullable;
import xyz.lychee.lagfixer.libs.annotations.ApiStatus.Internal;

public interface LegacyComponentSerializer extends ComponentSerializer<Component, TextComponent, String>, Buildable<LegacyComponentSerializer, Builder> {
   char SECTION_CHAR = '§';
   char AMPERSAND_CHAR = '&';
   char HEX_CHAR = '#';

   static @NotNull LegacyComponentSerializer legacySection() {
      return LegacyComponentSerializerImpl.Instances.SECTION;
   }

   static @NotNull LegacyComponentSerializer legacyAmpersand() {
      return LegacyComponentSerializerImpl.Instances.AMPERSAND;
   }

   static @NotNull LegacyComponentSerializer legacy(final char legacyCharacter) {
      if (legacyCharacter == 167) {
         return legacySection();
      } else {
         return legacyCharacter == '&' ? legacyAmpersand() : builder().character(legacyCharacter).build();
      }
   }

   static @Nullable LegacyFormat parseChar(final char character) {
      return LegacyComponentSerializerImpl.legacyFormat(character);
   }

   @NotNull
   static @NotNull Builder builder() {
      return new LegacyComponentSerializerImpl.BuilderImpl();
   }

   @NotNull TextComponent deserialize(final @NotNull String input);

   @NotNull String serialize(final @NotNull Component component);

   @PlatformAPI
   @Internal
   public interface Provider {
      @PlatformAPI
      @Internal
      @NotNull LegacyComponentSerializer legacyAmpersand();

      @PlatformAPI
      @Internal
      @NotNull LegacyComponentSerializer legacySection();

      @PlatformAPI
      @Internal
      @NotNull Consumer<Builder> legacy();
   }

   public interface Builder extends AbstractBuilder<LegacyComponentSerializer>, Buildable.Builder<LegacyComponentSerializer> {
      @NotNull
      @NotNull Builder character(final char legacyCharacter);

      @NotNull
      @NotNull Builder hexCharacter(final char legacyHexCharacter);

      @NotNull
      @NotNull Builder extractUrls();

      @NotNull
      @NotNull Builder extractUrls(final @NotNull Pattern pattern);

      @NotNull
      @NotNull Builder extractUrls(final @Nullable Style style);

      @NotNull
      @NotNull Builder extractUrls(final @NotNull Pattern pattern, final @Nullable Style style);

      @NotNull
      @NotNull Builder hexColors();

      @NotNull
      @NotNull Builder useUnusualXRepeatedCharacterHexFormat();

      @NotNull
      @NotNull Builder flattener(final @NotNull ComponentFlattener flattener);

      @NotNull LegacyComponentSerializer build();
   }
}
